/**
 * 
 */
package com.hdp.pi.service.kolon;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import au.com.bytecode.opencsv.CSVWriter;

import com.hdp.pi.domain.kolon.KolonMember;
import com.hdp.pi.domain.kolon.MongoPiShop;
import com.hdp.pi.domain.kolon.Product;
import com.hdp.pi.domain.kolon.Sale;
import com.hdp.pi.domain.member.Member;
import com.hdp.pi.dto.kolon.KolonMemberDTO;
import com.hdp.pi.repository.kolon.KolonMemberPreassignSnRepository;
import com.hdp.pi.repository.kolon.KolonMemberRepository;
import com.hdp.pi.repository.kolon.MongoPiShopRepository;
import com.hdp.pi.repository.kolon.ProductRepository;
import com.hdp.pi.repository.kolon.SaleRepository;
import com.hdp.pi.service.member.MemberService;
import com.hdp.pi.utils.kolon.FtpMemberUtil;
import com.hdp.pi.utils.kolon.FtpProductUtil;
import com.hdp.pi.utils.kolon.FtpSaleUtil;
import com.hdp.pi.utils.kolon.Util;

/**
 * @author yangyw
 *
 */
@Service
public class KolonMemberServiceImpl implements KolonMemberService {
	
	private static final Logger LOGGER = LoggerFactory
			.getLogger(KolonMemberServiceImpl.class);
	
	@Autowired
	private FtpMemberUtil ftpMemberUtil;

	@Autowired
	private FtpProductUtil ftpProductUtil;
	
	@Autowired
	private FtpSaleUtil ftpSaleUtil;
	
	@Autowired
	private KolonMemberRepository kolonMemberRepository;

	@Autowired
	private KolonMemberPreassignSnRepository kolonMemberPreassignSnRepository;
	
	@Autowired
	private ProductRepository productRepository;
	
	@Autowired
	private SaleRepository saleRepository;
	
	@Autowired
	private MongoPiShopRepository mongoPiShopRepository;
	
	@Autowired
	private MemberService memberService;
	
	@Override
	public KolonMember findOneByCsNo(String csNo) {
		return kolonMemberRepository.findFirstByCsNo(csNo);
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
	public void save(KolonMember kolonMember) {
		kolonMemberRepository.save(kolonMember);
	}

	@Override
	public List<KolonMember> findByUpdateTimeGreaterThan(Date updateTime) {
		return kolonMemberRepository.findByUpdateTimeGreaterThan(updateTime);
	}

	@Override
	public List<KolonMember> findByDataSourceAndUpdateTimeGreaterThan(
			Integer dataSource, Date updateTime) {
		return kolonMemberRepository.findByDataSourceAndUpdateTimeGreaterThan(
				dataSource, updateTime);
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
	public KolonMember matchData(String[] s) {
		KolonMemberDTO kolonMemberDTO = new KolonMemberDTO(s);
		String csNo = kolonMemberDTO.csNo;
		KolonMember kolonMember = null;
		
		//没有phone和csNo、cardNo，不处理
		if(!StringUtils.isNotEmpty(kolonMemberDTO.phone) && !StringUtils.isNotEmpty(csNo) && !StringUtils.isNotEmpty(kolonMemberDTO.cardNo)){
			return null;
		}
		
		if(StringUtils.isNotEmpty(csNo)){
			kolonMember = kolonMemberRepository.findFirstByCsNo(csNo);
		}
		if (kolonMember == null) { // 插入数据
			kolonMember = new KolonMember();
			kolonMember.csNo = csNo;
			kolonMember.cardNo = kolonMemberDTO.cardNo;
			kolonMember.phone = kolonMemberDTO.phone;
			kolonMember.lastName = kolonMemberDTO.lastName;
			kolonMember.firstName = kolonMemberDTO.firstName;
			kolonMember.email = kolonMemberDTO.email;
			kolonMember.status = kolonMemberDTO.status;
			kolonMember.bonus = kolonMemberDTO.bonus;
			kolonMember.lifecycle = kolonMemberDTO.lifecycle;
			kolonMember.customerType = kolonMemberDTO.customerType;
			kolonMember.dataSource = 2;
			kolonMember.birthday = Util.StringToDate(kolonMemberDTO.birthday);
			Date date = new Date();
			kolonMember.joinTime = date;
			kolonMember.updateTime = date;
			return kolonMember;
		} else { // 更新
			boolean flag = false;
			String oldCsNo = null;//是否更新了csno
			if(!kolonMemberDTO.csNo.equals(kolonMember.csNo)){//kolon和派加都注册了，以kolon为准
				oldCsNo = kolonMember.csNo;
				kolonMember.csNo = kolonMemberDTO.csNo;
				flag = true;
				
			}
			if(!kolonMemberDTO.cardNo.equals(kolonMember.cardNo)){//kolon和派加都注册了，以kolon为准
				kolonMember.cardNo = kolonMemberDTO.cardNo;
				flag = true;
			}
			
			if (!Util.isNotNull(kolonMember.email)
					|| !kolonMember.email.endsWith(kolonMemberDTO.email)) {
				kolonMember.email = kolonMemberDTO.email;
				flag = true;
			}
			if (!Util.isNotNull(kolonMember.firstName)
					|| !kolonMember.firstName
							.endsWith(kolonMemberDTO.firstName)) {
				kolonMember.firstName = kolonMemberDTO.firstName;
				flag = true;
			}
			if (!Util.isNotNull(kolonMember.lastName)
					|| !kolonMember.lastName
							.endsWith(kolonMemberDTO.lastName)) {
				kolonMember.lastName = kolonMemberDTO.lastName;
				flag = true;
			}
			if (kolonMemberDTO.status!=null) {
				kolonMember.status = kolonMemberDTO.status;
				flag = true;
			}
			if (kolonMemberDTO.bonus!=null) {
				kolonMember.bonus = kolonMemberDTO.bonus;
				flag = true;
			}
			if (StringUtils.isNotEmpty(kolonMemberDTO.lifecycle)) {
				kolonMember.lifecycle = kolonMemberDTO.lifecycle;
				flag = true;
			}
			if (kolonMemberDTO.customerType != null) {
				kolonMember.customerType = kolonMemberDTO.customerType;
				flag = true;
			}
			if (flag) {
				this.save(kolonMember);
				if(oldCsNo!=null){//csno被kolon覆盖
					kolonMemberPreassignSnRepository.setStatusWithKolonReplace(oldCsNo);
				}
			}
		}
		return null;
	}

	@Override
	public void syncMemberData() {
		String[]  listNames = ftpMemberUtil.listNames();
		for(String name : listNames){
			String sortName = name.substring(name.lastIndexOf("/") + 1, name.length());
			if(ftpMemberUtil.contains(sortName)){
				List<String[]> list = ftpMemberUtil.readCSVFile(sortName);
				LOGGER.info("正在导入" + sortName + "共" + list.size() + "条");
				if (list != null) {
					boolean firstRow = true;
					int i = 1;
					List<KolonMember> insertKolonMember = new ArrayList<KolonMember>();
					for (String[] row : list) {
						if (firstRow) {
							firstRow = false;
						} else {
							KolonMember insKolonMember = this.matchData(row);
							if(insKolonMember != null){
								insertKolonMember.add(insKolonMember);
							}
						}
						//大于等于200，批量插入
						if(insertKolonMember.size() >= 200){
							LOGGER.info("正在导入第【"+i+"】批");
							kolonMemberRepository.save(insertKolonMember);
							//清空
							insertKolonMember.clear();
							i++;
						}
					}
					//还有未插入的,继续保存
					if(insertKolonMember.size() > 0){
						kolonMemberRepository.save(insertKolonMember);
					}
				}
				LOGGER.info("导入完成");
				ftpMemberUtil.moveFile(sortName);
			}
		}
	}

	@Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
	private void saveProductAbsentIf(Product product){
		if(product == null || product.id == null){
			return;
		}
		Product p = productRepository.findOne(product.id);
		if(p == null){
			productRepository.save(product);
		}
	}
	
	@Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
	private void saveSale(Sale sale){
		if(sale == null){
			return;
		}
		if(sale.oId == null){
			return;
		}
		if(sale.oDate == null){
			return;
		}
		if(sale.customer == null){
			return;
		}
		if(sale.item == null){
			return;
		}
		if(sale.quantity == null){
			return;
		}
		if(sale.amount == null){
			return;
		}
		saleRepository.save(sale);
	}
	
	private Product newProduct(String[] row){
		Product p = new Product();
		try{
			p.id = Util.getByIndex(row, 0);
			p.title = Util.getByIndex(row, 1);
			p.category = Util.getByIndex(row, 2);
			p.image = Util.getByIndex(row, 3);
			p.link = Util.getByIndex(row, 4);
			p.avaliable = Util.getByIndex(row, 5);
			p.dept = Util.getByIndex(row, 6);
			p.seaons = Util.getByIndex(row, 7);
			p.planyy = Util.getByIndex(row, 8);
			p.color = Util.getByIndex(row, 9);
			p.price = Util.getByIndex(row, 10);
		}catch(Throwable e){
			e.printStackTrace();
		}
		return p;
	}
	
	private Sale newSale(String[] row){
		Sale s = new Sale();
		try{
			s.oId = Util.getByIndex(row, 0);
			s.oDate = Integer.valueOf(Util.getByIndex(row, 1));
			s.customer = Util.getByIndex(row, 2);
			s.item = Util.getByIndex(row, 3);
			s.quantity = Integer.valueOf(Util.getByIndex(row, 4));
			s.amount = Double.valueOf(Util.getByIndex(row, 5));
		}catch(Throwable e){
			e.printStackTrace();
		}
		return s;
	}
	
	@Override
	@Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
	public void syncProductData(){
		String[]  listNames = ftpProductUtil.listNames();
		for(String name : listNames){
			String sortName = name.substring(name.lastIndexOf("/") + 1, name.length());
			if(ftpProductUtil.contains(sortName)){
				List<String[]> list = ftpProductUtil.readCSVFile(sortName);
				if (list != null) {
					boolean firstRow = true;
					for (String[] row : list) {
						if (firstRow) {
							firstRow = false;
						} else {
							saveProductAbsentIf(newProduct(row));
						}
					}
				}
				ftpProductUtil.moveFile(sortName);
			}
		}
	}
	
	@Override
	@Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
	public void syncSaleData() {
		String[]  listNames = ftpSaleUtil.listNames();
		for(String name : listNames){
			String sortName = name.substring(name.lastIndexOf("/") + 1, name.length());
			if(ftpSaleUtil.contains(sortName)){
				List<String[]> list = ftpSaleUtil.readCSVFile(sortName);
				if (list != null) {
					boolean firstRow = true;
					for (String[] row : list) {
						if (firstRow) {
							firstRow = false;
						} else {
							saveSale(newSale(row));
						}
					}
				}
				ftpSaleUtil.moveFile(sortName);
			}
		}
	}
	
	@Override
	public Date sendData(Date time) {
		LOGGER.info("kolon发送数据 最后更新时间【" + time.toString() + "】");
		Date lastSendDate = time;
		Date tempSendDate = time;
		
		//kolonMembers2
		List<KolonMember> kolonMembers = kolonMemberRepository.findByDataSource2(2, time);
		//kolonMembers2
		List<KolonMember> kolonMembers2 = kolonMemberRepository.findByDataSource1(1, time);
		
		kolonMembers.addAll(kolonMembers2);
		
		List<String[]> addData = getDataList();
		List<String[]> updateData = getDataList();
		for(KolonMember km : kolonMembers){
			//查询Member
			Member member = null;
			if(Util.isNotNull(km.mid)){
				member = memberService.findById(km.mid);
			}
			if(km.dataSource == 2){
				updateData.add(convertDataUpdate(km, member, null));
			}else if(km.dataSource == 1 && km.joinTime.after(lastSendDate)){  //加入时间在发送时间之后，数据是新增的
				addData.add(convertDataAdd(km, member, null));
			}else{
				updateData.add(convertDataUpdate(km, member, null));
			}
			
			if(km.updateTime != null && tempSendDate.before(km.updateTime)){  //获取数据较后的更新时间
				tempSendDate = km.updateTime;
			}
		}
		LOGGER.info("kolon发送新增数据【" + (addData.size() - 1) + "】条");
		LOGGER.info("kolon发送更新数据【" + (updateData.size() - 1) + "】条");
		lastSendDate = tempSendDate;
		CSVWriter addWriter = ftpMemberUtil.getCSVWriter(1);
		writeDataToFile(addWriter, addData);
		CSVWriter updateWriter = ftpMemberUtil.getCSVWriter(2);
		writeDataToFile(updateWriter, updateData);
		//上传文件
		ftpMemberUtil.uploadFile();
		return lastSendDate;
	}
	
	private String[] convertDataUpdate(KolonMember km,Member member, String channelCode) {
		channelCode = getShopCode(member, channelCode);
		String sex = member == null || member.sex == null ? null : member.sex.toString();
		String birthday = member == null ? null : Util.dateToString(member.birthday);
		String provinceCode = member == null || member.provinceCode == null ? null : member.provinceCode.toString();
		String cityCode = member == null || member.cityCode == null ? null : member.cityCode.toString();
		String sectionCode = member == null || member.sectionCode == null ? null : member.sectionCode.toString();
		String address = member == null ? null : member.address;
		return new String[] { km.csNo, km.phone, km.email, km.wechatFirstName, km.wechatLastName, sex,
				birthday, provinceCode, cityCode, sectionCode,
				address, null, Util.dateToString(km.updateTime), channelCode};
	}

	
	private String[] convertDataAdd(KolonMember km,Member member, String channelCode) {
		channelCode = getShopCode(member, channelCode);
		String sex = member == null || member.sex == null ? null : member.sex.toString();
		String birthday = member == null ? null : Util.dateToString(member.birthday);
		String provinceCode = member == null || member.provinceCode == null ? null : member.provinceCode.toString();
		String cityCode = member == null || member.cityCode == null ? null : member.cityCode.toString();
		String sectionCode = member == null || member.sectionCode == null ? null : member.sectionCode.toString();
		String address = member == null ? null : member.address;
		return new String[] { km.csNo, km.phone, km.email, km.wechatFirstName, km.wechatLastName, sex,
				birthday, provinceCode, cityCode, sectionCode,
				address, Util.dateToString(km.joinTime), null, channelCode};
	}
	//获取门店编号
	private String getShopCode(Member member, String channelCode) {
		if(member != null && member.user != null && member.user.sourse != null && member.user.sourse.length() > 0){
			String sourse = member.user.sourse;
			//获取shopID
			String shopId = sourse.substring(sourse.lastIndexOf("_") + 1, sourse.length());
			if(Util.isNotNull(shopId) && shopId.equals("self")){
				return channelCode;
			}else{
				MongoPiShop mongoPiShop = mongoPiShopRepository.findFirstById(shopId);
				if(Util.isNotNull(mongoPiShop)){
					channelCode = Util.trimDecimal(mongoPiShop.shopCode);
				}else{
					LOGGER.warn("kolon获取门店编号出错，没有找到ID为【"+shopId+"】门店");
				}
			}
		}
		return channelCode;
	}
	

	private List<String[]> getDataList() {
		List<String[]> addData = new ArrayList<String[]>();
		String field = "CS_NO,Phone,Email,Wechat Last Name,Wechat First Name, Sex, Birthday, "
				+ "Province, City, District, Detail Address, Join Date, Binding Date, Channel Code";
		String[] header = field.split(",");
		addData.add(header);
		return addData;
	}
	
	public void writeDataToFile(CSVWriter writer, List<String[]> data){
		try {
			writer.writeAll(data);
			writer.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

}
